<?php

namespace Drupal\Tests\commerce_ticketing\Kernel;

use Drupal\commerce_order\Entity\Order;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_product\Entity\Product;
use Drupal\commerce_product\Entity\ProductVariation;
use Drupal\commerce_ticketing\Entity\CommerceTicket;

/**
 * Tests the ticket creation.
 *
 * @group commerce_ticketing
 */
class TicketCreationTest extends TicketKernelTestBase {

  /**
   * Tests ticket creation.
   */
  public function testTicketCreation() {
    $this->assertCount(0, $this->tickets);
    $this->addPayment();
    $tickets = CommerceTicket::loadMultiple();
    $this->assertCount(1, $tickets);
    $ticket = reset($tickets);
    $this->assertEquals($ticket->label(), 'Ticket ' . $this->variation->label());
  }

  /**
   * Tests ticket creation with a really long title.
   */
  public function testTicketCreationLongTitle() {
    $this->assertCount(0, $this->tickets);

    $customer = $this->createUser([], ['view own commerce_ticket', 'view own commerce_order', 'view commerce_product']);

    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = Order::create([
      'type' => 'default',
      'state' => 'draft',
      'mail' => $customer->getEmail(),
      'uid' => $customer->id(),
      'ip_address' => '127.0.0.1',
      'store_id' => $this->store->id(),
      'payment_gateway' => $this->paymentGateway->id(),
    ]);

    $variation = ProductVariation::create([
      'type' => 'ticket',
      'sku' => strtolower($this->randomMachineName()),
      'title' => $this->randomString(44),
      'price' => [
        'number' => '10.00',
        'currency_code' => 'USD',
      ],
      'status' => 1,
    ]);
    $variation->save();
    $product = Product::create([
      'type' => 'ticket',
      'title' => $this->randomMachineName(),
      'stores' => [$this->store],
      'variations' => [$variation],
    ]);
    $product->save();
    $variation = $this->reloadEntity($variation);

    // Add the item to the cart.
    $order_item = $this->orderItemStorage->createFromPurchasableEntity($variation);
    $order_item->save();
    $order->addItem($order_item);

    // Save and reload to have a fresh order.
    $order->save();
    $order = $this->reloadEntity($order);

    assert($order instanceof OrderInterface);
    $this->assertCount(1, $order->getItems());

    $this->addPayment($order);
    $tickets = CommerceTicket::loadMultiple();
    $this->assertCount(1, $tickets);
    $ticket = reset($tickets);
    $this->assertEquals($ticket->label(), substr($this->t('Ticket') . ' ' . $variation->label(), 0, 50));
  }

}
